-module(auto_player).
-behaviour(gen_server).

-export([start_link/0, init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2, code_change/3]).
-export([take_over/1, release/1, is_auto_playing/1]).

-record(state, {
    controlled_players = #{}, % Map: PlayerPid -> OriginalPlayer
    active_games = #{}       % Map: GamePid -> [{PlayerPid, Cards}]
}).

%% API
start_link() ->
    gen_server:start_link({local, ?MODULE}, ?MODULE, [], []).

take_over(PlayerPid) ->
    gen_server:call(?MODULE, {take_over, PlayerPid}).

release(PlayerPid) ->
    gen_server:call(?MODULE, {release, PlayerPid}).

is_auto_playing(PlayerPid) ->
    gen_server:call(?MODULE, {is_auto_playing, PlayerPid}).

%% Callbacks
init([]) ->
    {ok, #state{}}.

handle_call({take_over, PlayerPid}, _From, State = #state{controlled_players = Players}) ->
    case maps:is_key(PlayerPid, Players) of
        true ->
            {reply, {error, already_controlled}, State};
        false ->
            NewPlayers = maps:put(PlayerPid, {auto, os:timestamp()}, Players),
            {reply, ok, State#state{controlled_players = NewPlayers}}
    end;

handle_call({release, PlayerPid}, _From, State = #state{controlled_players = Players}) ->
    NewPlayers = maps:remove(PlayerPid, Players),
    {reply, ok, State#state{controlled_players = NewPlayers}};

handle_call({is_auto_playing, PlayerPid}, _From, State = #state{controlled_players = Players}) ->
    {reply, maps:is_key(PlayerPid, Players), State};

handle_call(_Request, _From, State) ->
    {reply, {error, unknown_call}, State}.

handle_cast({game_update, GamePid, PlayerPid, Cards}, State) ->
    NewState = update_game_state(GamePid, PlayerPid, Cards, State),
    maybe_play_turn(GamePid, PlayerPid, NewState),
    {noreply, NewState};

handle_cast(_Msg, State) ->
    {noreply, State}.

handle_info(_Info, State) ->
    {noreply, State}.

%% 内部函数

update_game_state(GamePid, PlayerPid, Cards, State = #state{active_games = Games}) ->
    GamePlayers = maps:get(GamePid, Games, []),
    UpdatedPlayers = lists:keystore(PlayerPid, 1, GamePlayers, {PlayerPid, Cards}),
    NewGames = maps:put(GamePid, UpdatedPlayers, Games),
    State#state{active_games = NewGames}.

maybe_play_turn(GamePid, PlayerPid, State) ->
    case should_play_turn(GamePid, PlayerPid, State) of
        true ->
            timer:sleep(rand:uniform(1000) + 500), % 添加随机延迟使行为更自然
            Play = calculate_auto_play(GamePid, PlayerPid, State),
            execute_play(GamePid, PlayerPid, Play);
        false ->
            ok
    end.

should_play_turn(GamePid, PlayerPid, #state{controlled_players = Players}) ->
    maps:is_key(PlayerPid, Players) andalso is_current_player(GamePid, PlayerPid).

is_current_player(GamePid, PlayerPid) ->
    % 从游戏服务器获取当前玩家
    case game_server:get_current_player(GamePid) of
        {ok, CurrentPlayer} -> CurrentPlayer =:= PlayerPid;
        _ -> false
    end.

calculate_auto_play(GamePid, PlayerPid, #state{active_games = Games}) ->
    GameState = maps:get(GamePid, Games, []),
    {_, Cards} = lists:keyfind(PlayerPid, 1, GameState),
    LastPlay = game_server:get_last_play(GamePid),
    case can_beat_play(Cards, LastPlay) of
        {true, Play} -> {play, Play};
        false -> pass
    end.

execute_play(_GamePid, _PlayerPid, pass) ->
    game_server:pass(_GamePid, _PlayerPid);
execute_play(_GamePid, _PlayerPid, {play, Cards}) ->
    game_server:play_cards(_GamePid, _PlayerPid, Cards).

can_beat_play(Cards, LastPlay) ->
    % 使用 card_rules 模块来找出可以压过上家的牌
    case card_rules:find_valid_plays(Cards, LastPlay) of
        [] -> false;
        [BestPlay|_] -> {true, BestPlay}
    end.

%% 处理代码热更新
code_change(_OldVsn, State, _Extra) ->
    {ok, State}.

%% 处理进程终止
terminate(_Reason, _State) ->
    ok.