-module(card_rules).
-export([validate_play/2, get_card_type/1, compare_cards/2]).

%% 验证出牌是否合法
validate_play(Cards, LastPlay) ->
    case {get_card_type(Cards), get_card_type(LastPlay)} of
        {invalid, _} -> false;
        {_, undefined} -> true; % 首次出牌
        {Type, Type} -> compare_cards(Cards, LastPlay);
        {bomb, _} -> true;
        {rocket, _} -> true;
        _ -> false
    end.

%% 获取牌型
get_card_type(Cards) ->
    case length(Cards) of
        0 -> undefined;
        1 -> single;
        2 -> check_pair(Cards);
        3 -> check_three(Cards);
        4 -> check_bomb_or_three_one(Cards);
        _ -> check_sequence(Cards)
    end.

%% 检查对子
check_pair([{_, N}, {_, N}]) -> pair;
check_pair(_) -> invalid.

%% 检查三张
check_three([{_, N}, {_, N}, {_, N}]) -> three;
check_three(_) -> invalid.

%% 检查炸弹或三带一
check_bomb_or_three_one(Cards) ->
    Grouped = group_cards(Cards),
    case Grouped of
        [{_, 4}|_] -> bomb;
        [{_, 3}, {_, 1}] -> three_one;
        _ -> invalid
    end.

%% 检查顺子等
check_sequence(Cards) ->
    case is_straight(Cards) of
        true -> straight;
        false -> check_other_types(Cards)
    end.

%% 检查其他牌型
check_other_types(Cards) ->
    Grouped = group_cards(Cards),
    case Grouped of
        [{_, 3}, {_, 2}] -> three_two;  % 三带二
        [{_, 4}, {_, 1}] -> four_one;   % 四带一
        [{_, 4}, {_, 2}] -> four_two;   % 四带二
        [{_, 3}, {_, 3}] -> airplane;   % 飞机
        _ -> invalid
    end.

%% 比较牌大小
compare_cards(Cards1, Cards2) ->
    case {get_card_type(Cards1), get_card_type(Cards2)} of
        {rocket, _} -> true;
        {bomb, OtherType} when OtherType =/= bomb -> true;
        {Same, Same} -> compare_value(Cards1, Cards2);
        _ -> false
    end.

%% 辅助函数 - 对牌进行分组
group_cards(Cards) ->
    Dict = lists:foldl(
        fun({_, N}, Acc) ->
            dict:update_counter(N, 1, Acc)
        end,
        dict:new(),
        Cards
    ),
    lists:sort(
        fun({_, Count1}, {_, Count2}) -> Count1 >= Count2 end,
        dict:to_list(Dict)
    ).

%% 辅助函数 - 判断是否为顺子
is_straight(Cards) ->
    CardValues = [{"3",3}, {"4",4}, {"5",5}, {"6",6}, {"7",7}, {"8",8}, {"9",9}, 
                 {"10",10}, {"J",11}, {"Q",12}, {"K",13}, {"A",14}, {"2",15}],
    Values = [proplists:get_value(N, CardValues) || {_, N} <- Cards],
    SortedValues = lists:sort(Values),
    case length(SortedValues) >= 5 of
        true ->
            lists:all(fun({A, B}) -> B - A =:= 1 end, 
                     lists:zip(SortedValues, tl(SortedValues)));
        false -> false
    end.

%% 比较相同牌型的大小
compare_value(Cards1, Cards2) ->
    CardValues = [{"3",3}, {"4",4}, {"5",5}, {"6",6}, {"7",7}, {"8",8}, {"9",9}, 
                 {"10",10}, {"J",11}, {"Q",12}, {"K",13}, {"A",14}, {"2",15},
                 {"小王",16}, {"大王",17}],
    Max1 = lists:max([proplists:get_value(N, CardValues) || {_, N} <- Cards1]),
    Max2 = lists:max([proplists:get_value(N, CardValues) || {_, N} <- Cards2]),
    Max1 > Max2.