-module(doudizhu_ai_strategy).

-export([
    evaluate_situation/2,
    choose_strategy/2,
    execute_strategy/3,
    calculate_move_score/3,
    analyze_hand_value/1
]).

-include("card_types.hrl").

-record(strategy_context, {
    game_stage,         % early_game | mid_game | end_game
    role,              % dizhu | nongmin
    hand_strength,     % 手牌强度评估
    control_level,     % 控制能力评估
    winning_prob,      % 胜率估计
    cards_remaining,   % 剩余手牌数
    opponent_cards     % 对手剩余牌数
}).

-record(hand_value, {
    singles = [],      % 单牌
    pairs = [],        % 对子
    triples = [],      % 三张
    sequences = [],    % 顺子
    bombs = [],        % 炸弹
    rockets = []       % 火箭
}).

%% 评估局势
evaluate_situation(GameState, PlayerState) ->
    HandCards = PlayerState#state.hand_cards,
    OpponentCards = get_opponent_cards(GameState),
    
    Context = #strategy_context{
        game_stage = determine_game_stage(GameState),
        role = PlayerState#state.role,
        hand_strength = evaluate_hand_strength(HandCards),
        control_level = calculate_control_level(GameState, PlayerState),
        winning_prob = estimate_winning_probability(GameState, PlayerState),
        cards_remaining = length(HandCards),
        opponent_cards = OpponentCards
    }.

%% 选择策略
choose_strategy(Context, Options) ->
    BaseStrategy = choose_base_strategy(Context),
    RefineStrategy = refine_strategy(BaseStrategy, Context, Options),
    adjust_strategy_for_endgame(RefineStrategy, Context).

%% 执行策略
execute_strategy(Strategy, GameState, PlayerState) ->
    HandCards = PlayerState#state.hand_cards,
    LastPlay = get_last_play(GameState),
    
    case Strategy of
        aggressive ->
            execute_aggressive_strategy(HandCards, LastPlay, GameState);
        conservative ->
            execute_conservative_strategy(HandCards, LastPlay, GameState);
        control ->
            execute_control_strategy(HandCards, LastPlay, GameState);
        explosive ->
            execute_explosive_strategy(HandCards, LastPlay, GameState)
    end.

%% 内部函数

%% 选择基础策略
choose_base_strategy(Context) ->
    case {Context#strategy_context.game_stage, Context#strategy_context.role} of
        {early_game, dizhu} when Context#strategy_context.hand_strength >= 0.7 ->
            aggressive;
        {early_game, nongmin} when Context#strategy_context.hand_strength >= 0.8 ->
            control;
        {mid_game, _} when Context#strategy_context.winning_prob >= 0.7 ->
            control;
        {end_game, _} when Context#strategy_context.cards_remaining =< 5 ->
            explosive;
        _ ->
            conservative
    end.

%% 细化策略
refine_strategy(BaseStrategy, Context, Options) ->
    case {BaseStrategy, Context#strategy_context.control_level} of
        {conservative, Control} when Control >= 0.8 ->
            control;
        {aggressive, Control} when Control =< 0.3 ->
            conservative;
        {Strategy, _} ->
            Strategy
    end.

%% 调整终盘策略
adjust_strategy_for_endgame(Strategy, Context) ->
    case Context#strategy_context.cards_remaining of
        N when N =< 3 ->
            case Context#strategy_context.winning_prob of
                P when P >= 0.8 -> aggressive;
                P when P =< 0.2 -> explosive;
                _ -> Strategy
            end;
        _ ->
            Strategy
    end.

%% 执行进攻策略
execute_aggressive_strategy(HandCards, LastPlay, GameState) ->
    HandValue = analyze_hand_value(HandCards),
    case LastPlay of
        none ->
            choose_aggressive_lead(HandValue);
        Play ->
            choose_aggressive_follow(HandValue, Play)
    end.

%% 执行保守策略
execute_conservative_strategy(HandCards, LastPlay, GameState) ->
    HandValue = analyze_hand_value(HandCards),
    case LastPlay of
        none ->
            choose_conservative_lead(HandValue);
        Play ->
            choose_conservative_follow(HandValue, Play)
    end.

%% 执行控制策略
execute_control_strategy(HandCards, LastPlay, GameState) ->
    HandValue = analyze_hand_value(HandCards),
    case LastPlay of
        none ->
            choose_control_lead(HandValue);
        Play ->
            choose_control_follow(HandValue, Play)
    end.

%% 执行爆发策略
execute_explosive_strategy(HandCards, LastPlay, GameState) ->
    HandValue = analyze_hand_value(HandCards),
    case LastPlay of
        none ->
            choose_explosive_lead(HandValue);
        Play ->
            choose_explosive_follow(HandValue, Play)
    end.

%% 选择进攻性首出
choose_aggressive_lead(HandValue) ->
    case find_best_lead_combination(HandValue) of
        {ok, Move} -> Move;
        none -> find_single_card(HandValue)
    end.

%% 选择进攻性跟牌
choose_aggressive_follow(HandValue, LastPlay) ->
    case find_minimum_greater_combination(HandValue, LastPlay) of
        {ok, Move} -> Move;
        none -> 
            case should_use_bomb(HandValue, LastPlay) of
                true -> find_smallest_bomb(HandValue);
                false -> pass
            end
    end.

%% 选择保守性首出
choose_conservative_lead(HandValue) ->
    case find_safe_lead(HandValue) of
        {ok, Move} -> Move;
        none -> find_smallest_single(HandValue)
    end.

%% 选择保守性跟牌
choose_conservative_follow(HandValue, LastPlay) ->
    case find_safe_follow(HandValue, LastPlay) of
        {ok, Move} -> Move;
        none -> pass
    end.

%% 选择控制性首出
choose_control_lead(HandValue) ->
    case find_control_combination(HandValue) of
        {ok, Move} -> Move;
        none -> find_tempo_play(HandValue)
    end.

%% 选择控制性跟牌
choose_control_follow(HandValue, LastPlay) ->
    case should_maintain_control(HandValue, LastPlay) of
        true -> find_minimum_greater_combination(HandValue, LastPlay);
        false -> pass
    end.

%% 选择爆发性首出
choose_explosive_lead(HandValue) ->
    case find_strongest_combination(HandValue) of
        {ok, Move} -> Move;
        none -> find_any_playable(HandValue)
    end.

%% 选择爆发性跟牌
choose_explosive_follow(HandValue, LastPlay) ->
    case find_crushing_play(HandValue, LastPlay) of
        {ok, Move} -> Move;
        none -> pass
    end.

%% 分析手牌价值
analyze_hand_value(Cards) ->
    GroupedCards = group_cards(Cards),
    #hand_value{
        singles = find_singles(GroupedCards),
        pairs = find_pairs(GroupedCards),
        triples = find_triples(GroupedCards),
        sequences = find_sequences(GroupedCards),
        bombs = find_bombs(GroupedCards),
        rockets = find_rockets(GroupedCards)
    }.

%% 计算出牌得分
calculate_move_score(Move, Context, LastPlay) ->
    BaseScore = calculate_base_score(Move),
    TempoScore = calculate_tempo_score(Move, Context),
    ControlScore = calculate_control_score(Move, Context),
    EfficiencyScore = calculate_efficiency_score(Move, Context),
    
    WeightedScore = BaseScore * 0.3 +
                    TempoScore * 0.2 +
                    ControlScore * 0.3 +
                    EfficiencyScore * 0.2,
    
    adjust_score_for_context(WeightedScore, Move, Context, LastPlay).

%% 辅助函数

group_cards(Cards) ->
    lists:foldl(fun(Card, Acc) ->
        {Value, _} = Card,
        maps:update_with(Value, fun(List) -> [Card|List] end, [Card], Acc)
    end, maps:new(), Cards).

find_singles(GroupedCards) ->
    maps:fold(fun(Value, [Card], Acc) ->
        [{Value, Card}|Acc];
        (_, _, Acc) -> Acc
    end, [], GroupedCards).

find_pairs(GroupedCards) ->
    maps:fold(fun(Value, Cards, Acc) ->
        case length(Cards) >= 2 of
            true -> [{Value, lists:sublist(Cards, 2)}|Acc];
            false -> Acc
        end
    end, [], GroupedCards).

find_triples(GroupedCards) ->
    maps:fold(fun(Value, Cards, Acc) ->
        case length(Cards) >= 3 of
            true -> [{Value, lists:sublist(Cards, 3)}|Acc];
            false -> Acc
        end
    end, [], GroupedCards).

find_sequences(GroupedCards) ->
    Values = lists:sort(maps:keys(GroupedCards)),
    find_consecutive_sequences(Values, GroupedCards, 5).

find_bombs(GroupedCards) ->
    maps:fold(fun(Value, Cards, Acc) ->
        case length(Cards) >= 4 of
            true -> [{Value, Cards}|Acc];
            false -> Acc
        end
    end, [], GroupedCards).

find_rockets(GroupedCards) ->
    case {maps:get(?CARD_JOKER_SMALL, GroupedCards, []),
          maps:get(?CARD_JOKER_BIG, GroupedCards, [])} of
        {[Small], [Big]} -> [{?CARD_JOKER_BIG, [Small, Big]}];
        _ -> []
    end.

find_consecutive_sequences(Values, GroupedCards, MinLength) ->
    find_sequences_of_length(Values, GroupedCards, MinLength, []).

find_sequences_of_length([], _, _, Acc) -> Acc;
find_sequences_of_length([V|Rest], GroupedCards, MinLength, Acc) ->
    case find_sequence_starting_at(V, Rest, GroupedCards, MinLength) of
        {ok, Sequence} ->
            find_sequences_of_length(Rest, GroupedCards, MinLength,
                                   [Sequence|Acc]);
        false ->
            find_sequences_of_length(Rest, GroupedCards, MinLength, Acc)
    end.

find_sequence_starting_at(Start, Rest, GroupedCards, MinLength) ->
    case collect_sequence(Start, Rest, GroupedCards, []) of
        Seq when length(Seq) >= MinLength ->
            {ok, {Start, Seq}};
        _ ->
            false
    end.

collect_sequence(Current, [Next|Rest], GroupedCards, Acc) when Next =:= Current + 1 ->
    case maps:get(Current, GroupedCards) of
        Cards when length(Cards) >= 1 ->
            collect_sequence(Next, Rest, GroupedCards, [hd(Cards)|Acc]);
        _ ->
            lists:reverse(Acc)
    end;
collect_sequence(Current, _, GroupedCards, Acc) ->
    case maps:get(Current, GroupedCards) of
        Cards when length(Cards) >= 1 ->
            lists:reverse([hd(Cards)|Acc]);
        _ ->
            lists:reverse(Acc)
    end.

calculate_base_score({Type, Value, _Cards}) ->
    BaseScore = Value * 10,
    TypeMultiplier = case Type of
        ?CARD_TYPE_ROCKET -> 100;
        ?CARD_TYPE_BOMB -> 80;
        ?CARD_TYPE_STRAIGHT -> 40;
        ?CARD_TYPE_STRAIGHT_PAIR -> 35;
        ?CARD_TYPE_PLANE -> 30;
        ?CARD_TYPE_THREE_TWO -> 25;
        ?CARD_TYPE_THREE_ONE -> 20;
        ?CARD_TYPE_THREE -> 15;
        ?CARD_TYPE_PAIR -> 10;
        ?CARD_TYPE_SINGLE -> 5
    end,
    BaseScore * TypeMultiplier / 100.

calculate_tempo_score(Move, Context) ->
    case Context#strategy_context.game_stage of
        early_game -> calculate_early_tempo(Move, Context);
        mid_game -> calculate_mid_tempo(Move, Context);
        end_game -> calculate_end_tempo(Move, Context)
    end.

calculate_control_score(Move, Context) ->
    BaseControl = case Move of
        {Type, _, _} when Type =:= ?CARD_TYPE_BOMB; 
                         Type =:= ?CARD_TYPE_ROCKET -> 1.0;
        {_, Value, _} when Value >= ?CARD_2 -> 0.8;
        _ -> 0.5
    end,
    BaseControl * Context#strategy_context.control_level.

calculate_efficiency_score(Move, Context) ->
    {Type, _, Cards} = Move,
    CardsUsed = length(Cards),
    RemainingCards = Context#strategy_context.cards_remaining - CardsUsed,
    Efficiency = CardsUsed / max(1, Context#strategy_context.cards_remaining),
    Efficiency * (1 + (20 - RemainingCards) / 20).

adjust_score_for_context(Score, Move, Context, LastPlay) ->
    case {Context#strategy_context.role, LastPlay} of
        {dizhu, none} -> Score * 1.2;
        {dizhu, _} -> Score * 1.1;
        {nongmin, _} -> Score
    end.

calculate_early_tempo(Move, Context) ->
    case Move of
        {Type, _, _} when Type =:= ?CARD_TYPE_SINGLE;
                         Type =:= ?CARD_TYPE_PAIR ->
            0.7;
        {Type, _, _} when Type =:= ?CARD_TYPE_STRAIGHT;
                         Type =:= ?CARD_TYPE_STRAIGHT_PAIR ->
            0.9;
        _ ->
            0.5
    end.

calculate_mid_tempo(Move, Context) ->
    case Move of
        {Type, _, _} when Type =:= ?CARD_TYPE_THREE_ONE;
                         Type =:= ?CARD_TYPE_THREE_TWO ->
            0.8;
        {Type, _, _} when Type =:= ?CARD_TYPE_PLANE ->
            0.9;
        _ ->
            0.6
    end.

calculate_end_tempo(Move, Context) ->
    case Move of
        {Type, _, _} when Type =:= ?CARD_TYPE_BOMB;
                         Type =:= ?CARD_TYPE_ROCKET ->
            1.0;
        {_, Value, _} when Value >= ?CARD_2 ->
            0.9;
        _ ->
            0.7
    end.