-module(room_manager).
-behaviour(gen_server).

-export([start_link/0, init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2, code_change/3]).
-export([create_room/2, list_rooms/0, join_room/2, leave_room/2, delete_room/1]).

-record(state, {
    rooms = #{} % Map: RoomId -> {RoomName, GamePid, Players, Status}
}).

-record(room, {
    id,
    name,
    game_pid,
    players = [], % [{PlayerPid, PlayerName}]
    status = waiting % waiting | playing
}).

%% API
start_link() ->
    gen_server:start_link({local, ?MODULE}, ?MODULE, [], []).

create_room(RoomName, CreatorPid) ->
    gen_server:call(?MODULE, {create_room, RoomName, CreatorPid}).

list_rooms() ->
    gen_server:call(?MODULE, list_rooms).

join_room(RoomId, PlayerPid) ->
    gen_server:call(?MODULE, {join_room, RoomId, PlayerPid}).

leave_room(RoomId, PlayerPid) ->
    gen_server:call(?MODULE, {leave_room, RoomId, PlayerPid}).

delete_room(RoomId) ->
    gen_server:call(?MODULE, {delete_room, RoomId}).

%% Callbacks
init([]) ->
    {ok, #state{}}.

handle_call({create_room, RoomName, CreatorPid}, _From, State = #state{rooms = Rooms}) ->
    RoomId = generate_room_id(),
    {ok, GamePid} = game_server:start_link(),
    NewRoom = #room{
        id = RoomId,
        name = RoomName,
        game_pid = GamePid,
        players = [{CreatorPid, get_player_name(CreatorPid)}]
    },
    NewRooms = maps:put(RoomId, NewRoom, Rooms),
    {reply, {ok, RoomId}, State#state{rooms = NewRooms}};

handle_call(list_rooms, _From, State = #state{rooms = Rooms}) ->
    RoomList = maps:fold(
        fun(RoomId, Room, Acc) ->
            [{RoomId, Room#room.name, length(Room#room.players), Room#room.status} | Acc]
        end,
        [],
        Rooms
    ),
    {reply, {ok, RoomList}, State};

handle_call({join_room, RoomId, PlayerPid}, _From, State = #state{rooms = Rooms}) ->
    case maps:find(RoomId, Rooms) of
        {ok, Room = #room{players = Players, status = waiting}} ->
            case length(Players) < 3 of
                true ->
                    NewPlayers = Players ++ [{PlayerPid, get_player_name(PlayerPid)}],
                    NewRoom = Room#room{players = NewPlayers},
                    NewRooms = maps:put(RoomId, NewRoom, Rooms),
                    {reply, {ok, Room#room.game_pid}, State#state{rooms = NewRooms}};
                false ->
                    {reply, {error, room_full}, State}
            end;
        {ok, #room{status = playing}} ->
            {reply, {error, game_in_progress}, State};
        error ->
            {reply, {error, room_not_found}, State}
    end;

handle_call({leave_room, RoomId, PlayerPid}, _From, State = #state{rooms = Rooms}) ->
    case maps:find(RoomId, Rooms) of
        {ok, Room = #room{players = Players}} ->
            NewPlayers = lists:keydelete(PlayerPid, 1, Players),
            NewRoom = Room#room{players = NewPlayers},
            NewRooms = maps:put(RoomId, NewRoom, Rooms),
            {reply, ok, State#state{rooms = NewRooms}};
        error ->
            {reply, {error, room_not_found}, State}
    end;

handle_call({delete_room, RoomId}, _From, State = #state{rooms = Rooms}) ->
    case maps:find(RoomId, Rooms) of
        {ok, Room} ->
            gen_server:stop(Room#room.game_pid),
            NewRooms = maps:remove(RoomId, Rooms),
            {reply, ok, State#state{rooms = NewRooms}};
        error ->
            {reply, {error, room_not_found}, State}
    end.

handle_cast(_Msg, State) ->
    {noreply, State}.

handle_info(_Info, State) ->
    {noreply, State}.

terminate(_Reason, _State) ->
    ok.

code_change(_OldVsn, State, _Extra) ->
    {ok, State}.

%% 内部辅助函数
generate_room_id() ->
    erlang:system_time().

get_player_name(PlayerPid) ->
    {ok, Name} = player:get_name(PlayerPid),
    Name.