-module(score_system).
-behaviour(gen_server).

-export([start_link/0, init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2, code_change/3]).
-export([get_score/1, update_score/3, get_leaderboard/0]).

-record(state, {
    scores = #{}, % Map: PlayerName -> {Score, Wins, Losses}
    leaderboard = []
}).

%% API
start_link() ->
    gen_server:start_link({local, ?MODULE}, ?MODULE, [], []).

get_score(PlayerName) ->
    gen_server:call(?MODULE, {get_score, PlayerName}).

update_score(PlayerName, GameResult, Points) ->
    gen_server:call(?MODULE, {update_score, PlayerName, GameResult, Points}).

get_leaderboard() ->
    gen_server:call(?MODULE, get_leaderboard).

%% Callbacks
init([]) ->
    {ok, #state{scores = #{}}}.

handle_call({get_score, PlayerName}, _From, State = #state{scores = Scores}) ->
    case maps:find(PlayerName, Scores) of
        {ok, ScoreData} ->
            {reply, {ok, ScoreData}, State};
        error ->
            {reply, {ok, {0, 0, 0}}, State}
    end;

handle_call({update_score, PlayerName, GameResult, Points}, _From, State = #state{scores = Scores}) ->
    {Score, Wins, Losses} = case maps:find(PlayerName, Scores) of
        {ok, {OldScore, OldWins, OldLosses}} ->
            {OldScore, OldWins, OldLosses};
        error ->
            {0, 0, 0}
    end,
    
    {NewScore, NewWins, NewLosses} = case GameResult of
        win -> {Score + Points, Wins + 1, Losses};
        loss -> {Score - Points, Wins, Losses + 1}
    end,
    
    NewScores = maps:put(PlayerName, {NewScore, NewWins, NewLosses}, Scores),
    NewLeaderboard = update_leaderboard(NewScores),
    
    {reply, {ok, {NewScore, NewWins, NewLosses}}, 
     State#state{scores = NewScores, leaderboard = NewLeaderboard}};

handle_call(get_leaderboard, _From, State = #state{leaderboard = Leaderboard}) ->
    {reply, {ok, Leaderboard}, State}.

handle_cast(_Msg, State) ->
    {noreply, State}.

handle_info(_Info, State) ->
    {noreply, State}.

terminate(_Reason, _State) ->
    ok.

code_change(_OldVsn, State, _Extra) ->
    {ok, State}.

%% 内部辅助函数
update_leaderboard(Scores) ->
    List = maps:to_list(Scores),
    SortedList = lists:sort(
        fun({_, {Score1, _, _}}, {_, {Score2, _, _}}) ->
            Score1 >= Score2
        end,
        List
    ),
    lists:sublist(SortedList, 10).